/**
 * Конвертирует номер версии, полученный в виде 1.2.2... в число типа float
 *
 * @param {String} v номер версии
 */
function versionToFloat(v){
	v = v.replace(',','.');
	var pointIndex = v.indexOf('.');
	var majorVersion = v.slice(0,pointIndex);
	var minorVersion = v.slice(pointIndex+1, v.length);
	minorVersion = minorVersion.replace('.','');

	return parseFloat(majorVersion+'.'+minorVersion);
}

/**
 * Отображение сообщения об ошибке. Показывать может только 1 сообщение, первое.
 *
 * @param {String} text текст сообщения об ошибке
 */
function gadgetErrorShow(text){
	if (!gadgetMessageDisplay) {
		gadgetMessageDisplay = 1;
		$('#gadgetMessageText').html(text);
		$('#closeMessage').get(0).onclick = function(){
			gadgetMessageHide()
		};
		$('#gadgetMessage').show();
		
		
		$('#newsDay').hide();
		$('#boxError').html(text);
		$('#boxError').show();
		
		dockStateChanged();
	}
}
/**
 * Скрытие сообщения об ошибке.
 * 
 */
function gadgetMessageHide(){
	gadgetMessageDisplay = 0;
	
	$('#boxError').html('');
	$('#boxError').hide();
	$('#newsDay').show();
	
	$('#gadgetMessage').hide();
	$('#gadgetMessageText').html('');
	dockStateChanged();
}

/**
 * Проверка: не вышла ли новая версия.
 * 
 */
function checkVersion(xml){
	//если что-то уже выводится, это ошибка и сейчас не до обновлений
	if (!gadgetMessageDisplay) {
		var localVersion = versionToFloat(System.Gadget.version);
		
		var currentVersion = $(xml).find('version').text();
		var updateURL = $(xml).find('url').text();
		currentVersion = versionToFloat(currentVersion);
		
		var now = new Date();


		if ((currentVersion > localVersion) && (showUpdateMessageAfter < now.getTime())) {
			var text = 'Обновить версию гаджета? <a href="'+updateURL+'" title="Установить свежую">Да</a> / <a href="#" title="Отложить вопрос" onclick="$(\'#closeMessage\').get(0).click()">Нет</a>';
			gadgetMessageDisplay = 1;
			$('#gadgetMessageText').html(text);
			$('#closeMessage').get(0).onclick = function(){
				var now = new Date();
				gadgetMessageHide();
				showUpdateMessageAfter = now.getTime()+1000*60*60*24*7; //откладываем на неделю
				System.Gadget.Settings.writeString('showUpdateMessageAfter', showUpdateMessageAfter);
			};
			$('#gadgetMessage').show();

			$('#newsDay').hide();
			$('#boxError').html(text);
			$('#boxError').show();
			
			dockStateChanged();
		}
	}
}



/**
 * Получение настроек или задание им значения по умолчанию
 * 
 */
function initSettings() {
	// устанавливаем значения по умолчанию при первом запуске гаджета
	realGadgetDocked = System.Gadget.Settings.readString('realGadgetDocked');
	if(realGadgetDocked == '') {
		realGadgetDocked = 1;
		System.Gadget.Settings.writeString('realGadgetDocked', realGadgetDocked);

	}
	
	gadgetTheme = System.Gadget.Settings.readString('gadgetTheme');
	if(gadgetTheme == '') {
		gadgetTheme = defaultTheme;
		System.Gadget.Settings.writeString('gadgetTheme', gadgetTheme);
	}

	newsNum = System.Gadget.Settings.readString('newsNum');
	if(newsNum == '') {
		newsNum = defaultNewsNum;
		System.Gadget.Settings.writeString('newsNum', newsNum);
	}

	showUpdateMessageAfter = System.Gadget.Settings.readString('showUpdateMessageAfter');
	if (showUpdateMessageAfter == '') {
		showUpdateMessageAfter = 0;
		System.Gadget.Settings.writeString('showUpdateMessageAfter', showUpdateMessageAfter);
	}
	else {
		showUpdateMessageAfter = parseInt(showUpdateMessageAfter);
	}
	
	var selectedTickersStr = System.Gadget.Settings.readString('selectedTickers');
	if (selectedTickersStr == '') {
		selectedTickers = defaultTickers;
		System.Gadget.Settings.writeString('selectedTickers', selectedTickers.join(' '));
	}
	else {
		selectedTickers = selectedTickersStr.split(' ');
	}

}

/**
 * Обработка закрытия страницы настроек
 * 
 */
function onSettingsClosed(e) {
	if (e.closeAction == e.Action.commit) {
		initSettings();
		applyTheme();
        dockStateChanged();
		showTickersShort();
		showNews();
    }
}

/**
 * Открытие флайаута
 * 
 */
function showFlyout(){
	System.Gadget.Flyout.show = !System.Gadget.Flyout.show;
    return false;
}

/**
 * Вычисление высоты гаджета
 * 
 */
function getGadgetHeight(is_flyout){
	var flyoutFilter;
	if (is_flyout==null || !is_flyout){
		flyoutFilter = 1;
	}
	else{
		flyoutFilter = 0;
	}

	var defaultHeightDocked = 169;
	var defaultHeightUndocked = 345;
	
	var dockedHeaderFooterHeight = 69;
	var dockedLineHeight = 20;
	var dockedMessageLineHeight = 10;

	var undockedHeaderFooterHeightCommon = 9;
	var undockedHeaderFooterHeightExtra = 29;
	var undockedTickerHeaderLineHeight = 19;
	var undockedTickerLineHeight = 16;
	var undockedNewsHeaderLineHeight = 26;
	var undockedNewsLineHeight = 27;
	var undockedMessageLineHeight = 18;
	var undockedExtraHeight = 18;
	var gHeight;
	if (realGadgetDocked) {
		gHeight = dockedMessageLineHeight*gadgetMessageDisplay + selectedTickers.length*dockedLineHeight+dockedHeaderFooterHeight;
	}
	else {
		gHeight =undockedMessageLineHeight*gadgetMessageDisplay+ undockedHeaderFooterHeightCommon+
		undockedHeaderFooterHeightExtra*flyoutFilter+
		(undockedTickerHeaderLineHeight*numUndockedTickerHeaders+undockedTickerLineHeight*numUndockedTickers)*flyoutFilter
		+undockedNewsHeaderLineHeight+undockedNewsLineHeight*newsNum+undockedExtraHeight;
	}
	return gHeight;

}

/**
 * Изменение режима гаджета
 * @param {mixed} is_flyout Если true, изменения применяем к флайауту
 */
function dockStateChanged(is_flyout) {

	if (is_flyout==null || !is_flyout){
		is_flyout = false;
		realGadgetDocked = System.Gadget.docked;
		System.Gadget.Settings.writeString('realGadgetDocked', (System.Gadget.docked?1:0));
	}
	else{
		realGadgetDocked = false;
	}


    if (realGadgetDocked) {
		var width = '138px';
		var height = getGadgetHeight()+'px';
		$('#background').css('width', width)
						.css('height', height);
		setBackground( height, 'skin/img/blocks_'+gadgetTheme+'_top.png', 'skin/img/blocks_'+gadgetTheme+'_big.png', 'skin/img/blocks_'+gadgetTheme+'_small.png', 'skin/img/blocks_'+gadgetTheme+'_bottom.png');

        $(document.body).css('width', width)
						.css('height', height);
		$('#fullContainerBig').hide();
		$('#fullContainer').show();
    } else {

		var width = '290px';
		var height = getGadgetHeight(is_flyout)+'px';
		
		$('#background').css('width', width)
						.css('height', height);
		if (is_flyout) {
			setBackground(height, 'skin/img/blocks_fly_' + gadgetTheme + '_top_B.png', 'skin/img/blocks_' + gadgetTheme + '_B_big.png', 'skin/img/blocks_' + gadgetTheme + '_B_small.png', 'skin/img/blocks_' + gadgetTheme + '_bottom_B.png');
		}
		else {
			setBackground(height, 'skin/img/blocks_' + gadgetTheme + '_top_B.png', 'skin/img/blocks_' + gadgetTheme + '_B_big.png', 'skin/img/blocks_' + gadgetTheme + '_B_small.png', 'skin/img/blocks_' + gadgetTheme + '_bottom_B.png');
		}
		$(document.body).css('width', width)
						.css('height', height);
		$('#fullContainer').hide();
		$('#fullContainerBig').show();
    }
}

   /**  
    * @description  
    * changes the background size and images  
    *  
    * @param {Number} height the new gadget height  
    * @param {String} headerUrl the new gadget header image url 
    * @param {String} midleSliceUrlBig the new gadget middle repeat slice N>1px height   
    * @param {String} midleSliceUrlSmall the new gadget middle repeat slice 1px height  
    * @param {String} footerUrl the new gadget footer image url  
    */  
function setBackground(height, headerUrl, midleSliceUrlBig, midleSliceUrlSmall, footerUrl){

    // resize part   

    // parses the height as integer, just in case   
    _nHeight = parseInt(height, 10);
    _headerUrl = headerUrl;
	_midleSliceUrlBig=midleSliceUrlBig;
	_midleSliceUrlSmall=midleSliceUrlSmall;
	_footerUrl=footerUrl;
    
    
    
    // fill the background image   
    
    //load the images to get the sizes   
    _headerImg = new Image();
    _sliceImgBig = new Image();
    _sliceImgSmall = new Image();
    _footerImg = new Image();
	
    bgSettingFinished = false;
    
    _headerImg.src = headerUrl;
    _sliceImgBig.src = midleSliceUrlBig;
    _sliceImgSmall.src = midleSliceUrlSmall;
    _footerImg.src = footerUrl;
    
    
    
    _headerImg.onload = function(){
        continueSetting();
    };
    _sliceImgBig.onload = function(){
        continueSetting();

    };
    _sliceImgSmall.onload = function(){
        continueSetting();
    };
    _footerImg.onload = function(){
        continueSetting();
    };
    continueSetting();
    
}
/**
 * Обработка закрытия страницы настроек
 * 
 */
function continueSetting(){


    if (!bgSettingFinished &&
		(_headerImg!=null && _headerImg.height) &&
		 (_sliceImgBig!=null && _sliceImgBig.height) &&
		 (_sliceImgSmall!=null && _sliceImgSmall.height) && 
		 (_footerImg!=null && _footerImg.height) ) {

        
        _background.removeObjects();
        
        _body.style.height = _nHeight + 'px';
        _background.style.height = _nHeight + 'px';
        
        var _headerHeight, _sliceHeight, _footerHeight;
        // set the sizes   
        _headerHeight = _headerImg.height;
        _sliceHeightBig = _sliceImgBig.height;
        _sliceHeightSmall = _sliceImgSmall.height;
        _footerHeight = _footerImg.height;
        
        var _top = 0;
        
        // add the header   
        _background.addImageObject(_headerUrl, 0, _top);
        _top += _headerHeight;
        
        //calculate the middle part height   
        var _middlepartHeight = _nHeight - _headerHeight - _footerHeight;
        var _middleFilledHeight = 0;
        if (_middlepartHeight >= _sliceHeightBig) {
            //часть фона замостим большими картинками
            var bigPartHeight = _middlepartHeight;
            if ((_middlepartHeight % _sliceHeightBig) != 0) {
                bigPartHeight = Math.floor(_middlepartHeight / _sliceHeightBig) * _sliceHeightBig;
            }
            
            for (var _i = 0; _i < bigPartHeight; _i += _sliceHeightBig) {
                _background.addImageObject(_midleSliceUrlBig, 0, _top);
                _top += _sliceHeightBig;
                _middleFilledHeight += _sliceHeightBig;
            }
        }
        
        if (_middleFilledHeight < _middlepartHeight) {
            //а что осталось - маленькими
            for (var _i = _middleFilledHeight; _i < _middlepartHeight; _i += _sliceHeightSmall) {
                _background.addImageObject(_midleSliceUrlSmall, 0, _top);
                _top += _sliceHeightSmall;
            }
        }
        
        // now add the footer   
        _background.addImageObject(_footerUrl, 0, _top);
        bgSettingFinished = true;
        
        _headerImg = null;
        _sliceImgBig = null;
        _sliceImgSmall = null;
        _footerImg = null;
    }
}
/**
 * Задание темы оформления гаджета в соответствии с текущими настройками
 * 
 */
function applyTheme(){
	$('#fullContainer').removeClass('greenBG blackBG');
	$('#fullContainerBig').removeClass('greenBG blackBG');
	
	$('#fullContainer').addClass(gadgetTheme);
	$('#fullContainerBig').addClass(gadgetTheme);
}
 
/**
 * Обработчик успешной загрузки короткого списка тикеров
 * 
 * @param {String} html загруженные данные
 */
function grabTickersShortSuccess (html){

	html = cleanupHTML(html);
	currentTickersShortHTML = html.replace(/(\n|\r)/mg, "");
	setPersistentSetting('tickersShortHTML', currentTickersShortHTML);
	
	showTickersShort();
}
/**
 * Обработчик ошибки при загрузке короткого списка тикеров
 * 
 * @param {Object} XMLHttpRequest запрос
 * @param {String} textStatus строка с ошибкой
 * @param {Object} errorThrown исключение
 */
function grabTickersShortError (XMLHttpRequest, textStatus, errorThrown){
	gadgetErrorShow('<a href="#" onclick="grabData()" title="Обновить">Ошибка получения данных</a>');
    getPersistentSetting('tickersShortHTML', function(html){
		if (html != null) {
			currentTickersShortHTML = html;
			showTickersShort();
		}
    });
}

/**
 * Отображение короткого списка тикеров
 * 
 */
function showTickersShort(){
	var html = currentTickersShortHTML;

	$("#subBox").hide();
    $("#subBox > table").remove();
	$("#subBox").prepend(html);
	
	//сохраним серию пар id_тикера-название
	//надо следить, чтобы длина строки была не больше 2048 символов
	System.Gadget.Settings.writeString('tickersList', '');
	$("#subBox tr").each(function(index){
		var currTickers = System.Gadget.Settings.readString('tickersList');
		currTickers+=((index>0)?'~':'')+$(this).attr('id')+'='+$(this).children().filter('.nameInd').text();
		System.Gadget.Settings.writeString('tickersList', currTickers);
	
	});
	
	$("#subBox  tr:not(#"+selectedTickers.join(",#")+")").remove();
	$("#subBox").show();
}

/**
 * Обработчик успешной загрузки новостей
 * 
 * @param {String} html загруженные данные
 */
function grabNewsSuccess (html){

	html = cleanupHTML(html);
	currentNewsHTML = html.replace(/(\n|\r)/mg, "");
	setPersistentSetting('newsHTML', html);
	showNews();
}

/**
 * Обработчик ошибки при загрузке новостей
 * 
 * @param {Object} XMLHttpRequest запрос
 * @param {String} textStatus строка с ошибкой
 * @param {Object} errorThrown исключение
 */
function grabNewsError (XMLHttpRequest, textStatus, errorThrown){

    getPersistentSetting('newsHTML', function(html){
		if (html != null) {
			currentNewsHTML = html;
			showNews();
		}
    });
}

/**
 * Отображение списка новостей
 * 
 */
function showNews(){
	var html = currentNewsHTML;
	var filteredNews = $(html).not('.item:gt('+(newsNum-1)+')');
    $("#newsBox").empty();
	$("#newsBox").append(filteredNews);
}

/**
 * Обработчик успешной загрузки полного списка тикеров
 * 
 * @param {String} html загруженные данные
 */
function grabTickersAllSuccess (html){

	html = cleanupHTML(html);
	currentTickersAllHTML = html.replace(/(\n|\r)/mg, "");

	setPersistentSetting('tickersAllHTML', currentTickersAllHTML);
	showTickersAll();
}
/**
 * Обработчик ошибки при загрузке полного списка тикеров
 * 
 * @param {Object} XMLHttpRequest запрос
 * @param {String} textStatus строка с ошибкой
 * @param {Object} errorThrown исключение
 */
function grabTickersAllError (XMLHttpRequest, textStatus, errorThrown){

    getPersistentSetting('tickersAllHTML', function(html){
		if (html != null) {
			currentTickersAllHTML = html;
			showTickersAll();
		}
    });
}
/**
 * Отображение полного списка тикеров
 * 
 */
function showTickersAll(){
	var html = currentTickersAllHTML;
	numUndockedTickerHeaders = $(html).find('tr:has(th)').size();
	numUndockedTickers = $(html).find('.tickerLine').size();

    $("#rateBox").empty();
	$("#rateBox").append(html);
}

/**
 * Очистка HTML от потенциально опасных вещей
 * 
 * @param {String} html загруженные данные
 * @return {String} html очищенный код данные
 */
function cleanupHTML(html){
	html = html.replace(/script/ig, '');
	html = html.replace(/iframe/ig, '');

	return html;
}

/**
 * Отправка запросов на получение данных (новостей и тикеров)
 * 
 */  
function grabData(){

	$.ajax({
	  url: tickersShortDataURL+'?r='+Math.random(),
	  cache: false,
	  success: grabTickersShortSuccess,
	  error: grabTickersShortError,
	  timeout: ajaxTimeout
	});

	$.ajax({
	  url: newsDataURL+'?r='+Math.random(),
	  cache: false,
	  success: grabNewsSuccess,
	  error: grabNewsError,
	  timeout: ajaxTimeout
	});

	$.ajax({
	  url: tickersAllDataURL+'?r='+Math.random(),
	  cache: false,
	  success: grabTickersAllSuccess,
	  error: grabTickersAllError,
	  timeout: ajaxTimeout
	});
}

var gadgetTheme;
var realGadgetDocked;

var _headerImg, _sliceImgBig, _sliceImgSmall, _footerImg; 
var _headerUrl, _midleSliceUrlBig, _midleSliceUrlSmall, _footerUrl;  

var bgSettingFinished = false;
var _body;   
var _background;   
var _nHeight;

var newsNum; //количество показываемых новостей

var dataRefreshTimer;
var currentNewsHTML;
var currentTickersAllHTML;
var currentTickersShortHTML;

var numUndockedTickerHeaders=3; //количество заголовков в "развернутом" списке тикеров (типа "Мировые индексы")
var numUndockedTickers=9; //количество тикеров в "развернутом" списке тикеров
var selectedTickers = new Array(); //выбранные тикеры, которые будут показываться в свёрнутом списке
var gadgetMessageDisplay = 0; //показывать ли сообщение
var showUpdateMessageAfter = 0; //после какого времени можно показывать сообщение об обновлении